//----------------------------------------------------------------------------------------------------//

/* 
 *       This Code Was Created By Jan Koci (Kwan) 2002
 *       Visit My Site (GLART) At http://koci.opengl.cz 
 */

//----------------------------------------------------------------------------------------------------//

/* 
 * KCamera.cpp
 * 
 * Zdrojov soubor pohybu kamery
 * 
 *
 */

//----------------------------------------------------------------------------------------------------//

#include "stdafx.h"		// Pouvej hlavikov soubor pedkopilovanch hlavikovch soubor 

//----------------------------------------------------------------------------------------------------//

KCamera::KCamera()		// Konstruktor tdy KCamera
{
	// Nastav tyto poten hodnoty kamery
	Position.x = 0.0;		// X pozice kamery
	Position.y = 0.0;		// Y pozice kamery
	Position.z = 10.0;		// Z pozice kamery

	Angle.x = 0;			// X hel rotace kamery
	Angle.y = 0;			// Y hel rotace kamery
	Angle.z = 0;			// Z hel rotace kamery


	View.x =  0.0;			// X pozice bodu, kam kouk kamera
	View.y =  0.0;			// Y pozice bodu, kam kouk kamera
	View.z =  -1.0;			// Z pozice bodu, kam kouk kamera

	Up.x =  0.0;			// X pozice Up vektoru
	Up.y =  1.0;			// Y pozice Up vektoru
	Up.z =  0.0;			// Z pozice Up vektoru

	speed = 0.1f;			// Rychlost pohybu kamery
}

//----------------------------------------------------------------------------------------------------//

KCamera::~KCamera()		// Destruktor tdy KCamera
{
}

//----------------------------------------------------------------------------------------------------//

// Vypotej pozici bodu, na kter kouk kamera
void KCamera::RotateView(float angle, float x, float y, float z)
{
	KVector3 vNewView;

	// Vypotej vektor pohledu kamery 
	KVector3 vView = View - Position;		

	// Vytvo si hodnoty sin, cos, hlu rotace
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// Najdi novou x pozici pro nov bod, na kter kouk kamera
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)		* vView.x;
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y;
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z;

	// Najdi novou y pozici pro nov bod, na kter kouk kamera
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x;
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)		* vView.y;
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z;

	// Najdi novou z pozici pro nov bod, na kter kouk kamera
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x;
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y;
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)		* vView.z;

	// Piti k pozici kamery nov rotan vektor, m vypot nov bod na kter kouk kamera
	View = Position + vNewView;
}

//----------------------------------------------------------------------------------------------------//

// Pohni kamerou vped, nebo vzad podle hodnoty speed (+ nebo -) 
void KCamera::MoveCamera(float speed)
{
	KVector3 vVector = View - Position;	// Vypotej vektor pohledu kamery
	vVector = Normalize(vVector);		// Normalizuj vektor

	Position.x += vVector.x * speed;	// Vypotej novou hodnotu pozice bodu X kamery, podle vektoru pohledu a rychlosti posunut
	Position.y += vVector.y * speed;	// Vypotej novou hodnotu pozice bodu Y kamery, podle vektoru pohledu a rychlosti posunut
	Position.z += vVector.z * speed;	// Vypotej novou hodnotu pozice bodu Z kamery, podle vektoru pohledu a rychlosti posunut
	View.x += vVector.x * speed;		// Vypotej novou hodnotu vektoru pohledu X, podle vektoru pohledu a rychlosti posunut
	View.y += vVector.y * speed;		// Vypotej novou hodnotu vektoru pohledu Y, podle vektoru pohledu a rychlosti posunut
	View.z += vVector.z * speed;		// Vypotej novou hodnotu vektoru pohledu Z, podle vektoru pohledu a rychlosti posunut
}

//----------------------------------------------------------------------------------------------------//

// Posu kameru doprava, nebo doleva podle hodnoty speed (+ nebo -)
void KCamera::StrafeCamera(float speed)
{	
	// Vypotej novou hodnotu pozice kamery, podle vektoru posunut a rychlosti posunut
	Position.x += Strafe.x * speed;
	Position.z += Strafe.z * speed;

	// Vypotej novou hodnotu vektoru pohledu, podle vektoru posunut a rychlosti posunut
	View.x += Strafe.x * speed;
	View.z += Strafe.z * speed;
}

//----------------------------------------------------------------------------------------------------//

// Rotuj kamerou pomoc myi (First person view)
void KCamera::SetViewByMouse()
{
	POINT mousePos;									// Pozice kurzoru myi
	int middleX = SCREEN_WIDTH  >> 1;				// X sted obrazovky. Bitov posun doprava, zpsob dlen dvma ky obrazovky
	int middleY = SCREEN_HEIGHT >> 1;				// Y sted obrazovky. Bitov posun doprava, zpsob dlen dvma vky obrazovky
	float angleY = 0.0f;							// Rotace kolem osy Y, co zpsob otoen kamery doprava a doleva
	float angleZ = 0.0f;							// Smr pohledu kamery nahoru a dolu
	static float currentRotX = 0.0f;				// Rotace kamery v X smru
	
	// Zskej X,Y pozici kurzoru myi
	GetCursorPos(&mousePos);
	
	// Pokud je kurzor stle uprosted okna, nic by se nemlo hbat, proto opus funkci
	if( (mousePos.x == middleX) && (mousePos.y == middleY) ) return;

	// Jinak vra kurzor zpt do stedu okna
	SetCursorPos(middleX, middleY);							

	// Zskej hel Y a Z podle smru pohybu kurzoru 
	angleY = (float)( (middleX - mousePos.x) ) / 500.0f;		
	angleZ = (float)( (middleY - mousePos.y) ) / 500.0f;		

	// Nastav stvjc rotaci kamery v X smru
	// Hodnotu pak pouj k tomu, aby kamera v X smru nerotovala 360 stupnu, 
	// ale aby se zastavovala u stropu a u podlahy
	currentRotX -= angleZ;  

	// Jestlie je stvjc rotace X (kter je v radianech) vt ne 1, tak vra hodnotu zpt na 1
	if(currentRotX > 1.0f)
		currentRotX = 1.0f;
	// Jestlie je stvjc rotace X (kter je v radianech) men ne -1, tak vra hodnotu zpt na -1
	else if(currentRotX < -1.0f)
		currentRotX = -1.0f;
	// Jinak rotuj dle 
	else
	{
		// Aby rotovala kamera nahoru a dolu, mus vypotat 
		// kolm vektor na vektor pohledu kamery. 
		KVector3 vAxis = Cross(View - Position, Up);	// Vypotej kolm vektor osy na vektor pohledu kamery
		vAxis = Normalize(vAxis);						// Normalizuj vektor osy

		RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z);	// Rotuj nahoru a dolu v nov ose
		RotateView(angleY, 0, 1, 0);					// Rotuj doleva a doprava
	}
}

//----------------------------------------------------------------------------------------------------//

void KCamera::GoForward()		// Pohni kamerou dopedu
{
	MoveCamera(speed);
}

//----------------------------------------------------------------------------------------------------//

void KCamera::GoBackward()		// Pohni kamerou dozadu
{
	MoveCamera(-speed);
}

//----------------------------------------------------------------------------------------------------//

void KCamera::GoUp()					// Pohni kamerou nahoru
{
    Position.y += speed;				// Pohni pozic kamery
	View.y += speed;					// A zrove pohni bodem, na kter kamera kouk
}

//----------------------------------------------------------------------------------------------------//

void KCamera::GoDown()					// Pohni kamerou dolu
{
	Position.y -= speed;	// Pohni pozic kamery
	View.y -= speed;			// A zrove pohni bodem, na kter kamera kouk
}

//----------------------------------------------------------------------------------------------------//

void KCamera::StrafeLeft()							// Pohni kamerou doleva
{
	KVector3 vCross = Cross(View - Position, Up);	// Vypotej kolm vektor osy na vektor pohledu kamery
	Strafe = Normalize(vCross);						// Normalizuj vektor posunut
	StrafeCamera(-speed);							// Pohni kamerou o hodnotu rychlosti kamery
}

//----------------------------------------------------------------------------------------------------//

void KCamera::StrafeRight()		// Pohni kamerou doprava
{
	KVector3 vCross = Cross(View - Position, Up);	// Vypotej kolm vektor osy na vektor pohledu kamery
	Strafe = Normalize(vCross);						// Normalizuj vektor posunut
	StrafeCamera(speed);							// Pohni kamerou o hodnotu rychlosti kamery
}

//----------------------------------------------------------------------------------------------------//

void KCamera::TurnLeft()			// Oto kamerou doleva
{
	Angle.y = speed;				// Nastav rychlost rotace
	RotateView(Angle.y, 0, 1, 0);	// Rotuj bodem, na kter kouk kamera
}

//----------------------------------------------------------------------------------------------------//

void KCamera::TurnRight()			// Oto kamerou doleva
{
	Angle.y = -speed;				// Nastav rychlost rotace
	RotateView(Angle.y, 0, 1, 0);	// Rotuj bodem, na kter kouk kamera
}

//----------------------------------------------------------------------------------------------------//

void KCamera::TurnUp()		// Oto kamerou nahoru
{
/*
	KVector3 vAxis = Cross(View - Position, Up);	// Vypotej kolm vektor osy na vektor pohledu kamery
	vAxis = Normalize(vAxis);						// Normalizuj vektor osy
	Angle.z = speed;								// Nastav rychlost rotace
	RotateView(Angle.z, vAxis.x, vAxis.y, vAxis.z);	// Rotuj bodem, na kter kouk kamera
*/
}

//----------------------------------------------------------------------------------------------------//

void KCamera::TurnDown()		// Oto kamerou dolu
{
/*
	KVector3 vAxis = Cross(View - Position, Up);	// Vypotej kolm vektor osy na vektor pohledu kamery
	vAxis = Normalize(vAxis);						// Normalizuj vektor osy
	Angle.z = -speed;								// Nastav rychlost rotace
	RotateView(Angle.z, vAxis.x, vAxis.y, vAxis.z);	// Rotuj bodem, na kter kouk kamera
*/
}

//----------------------------------------------------------------------------------------------------//

void KCamera::Look()
{
	// Rotuj pomoc myi
	SetViewByMouse();
	
	// Pohybuj a rotuj kamerou
	gluLookAt(Position.x, Position.y, Position.z,	// Pozice bodu kamery
			  View.x, View.y, View.z,				// Pozice bodu, kam kouk kamera	
			  Up.x, Up.y, Up.z);					// Pozice Up vektoru Y=1;
}

//----------------------------------------------------------------------------------------------------//

